%This is the script used for the data processing of the meteorological data
%in MATLAB. The raw data file from Draugen is used to illustrate what
%has been done


%% INSERT VALUES FOR HS,CUT-OUT
close all
clear all
cutoffbolge = 9.0; %Cut-out wave height (called cutoff in whole script although it should be cut-out)
recutinn = 7.0; %Re cut-in wave height for strategy 1


%% LOADING DATA INTO MATLAB
data = textread('draugendata.txt'); %This file contains many parameters which are not used in the data processing (wave period, maximum wave height etc.)

%Loading in dates
m=data(:,2);
m(1)
m(end)
l = num2str(m);
k= datenum(l,'yyyymmddHHMM');

%Loading in wind speed data in vector u
u = data(:,5);
u(u==0)=NaN;

%%calculation wind speed in hub height
malehoyde = 78; %measuring height
navhoyde = 90; % hub height
skjaer = 0.1; % shear exponent
reduksjon= 0.77; %reduction factor
u = u./(reduksjon)*(navhoyde/malehoyde)^skjaer; 
u(1)=0; %setting first wind speed to zero (due to later removing of power because of the cut-out wave height

%Loading in wave height data in vector H
H= data(:,8);
H(H==0)=NaN;

%Interpolating missing values by cubic interpolation in MATLAB
u(isnan(u)) = interp1(find(~isnan(u)), u(~isnan(u)), find(isnan(u)), 'cubic');
H(isnan(H)) = interp1(find(~isnan(H)), H(~isnan(H)), find(isnan(H)),'cubic');


%% PLOTTING WIND SPEED AND WAVE HEIGHT
%Plotting time series of u and H
figure(101)
plot(k,u,k,H)
datetick
xlabel('Time [days]')
ylabel('Signficant Wave Height [m] ,Wind Speed [m/s]')
title('Time series of wind speed and wave height')

%Histograms of u and H are made to see distribution of wind speed and wave
%heights
x = 0.5:1:(round(max(u))+0.5);
y = 0.5:1:(round(max(H))+0.5);
figure(2)
hist(u,x)
xlabel('Wind Speed [m/s]')
ylabel('Counts')
title('Wind speed histogram/distribution')
figure(3)
hist(H,y)
xlabel('Signficant Wave Height [m]')
ylabel( 'Counts')
title('Wave height histogram/distribution')



%I plot u vs. H to see how wind and wave correlates with each other
figure(9)
plot(H,u,'d')
xlabel('Hs, Significant Wave Height [m]');
ylabel('u, Wind speed [m/s]');
title('Coherence of wind speed and wave height')

%The correlation can be visualized with a 3D-histogram, counting the number
%of times a certain wind speed and wave height occurs
X = [  u H ];
X1 = [H u];
H1bins = 0:1:15;
u1bins = 0:1:50;
figure(6)
r1 = hist3(X,'Edges',{u1bins H1bins });
hist3(X1,'Edges',{ H1bins u1bins });
hold on
xlabel('Hs, Significant Wave Height [m]');
ylabel('u, Wind speed [m/s]');
zlabel('Frequency');
title('Joint histogram of wind and waves');
set(gcf,'renderer','opengl');
set(get(gca,'child'),'FaceColor','interp','CDataMode','auto');

%The 3D-histogram can be plotted as a surface plot, smoothing out the
%surfaces
figure(11)
Z = surf(r1);
xlabel('Hs, Significant Wave Height [m]');
ylabel('u, Wind speed [m/s]');
zlabel('Counts');
title('Surface plot of Joint histogram')

%The same plot is made, but the z-axis now shows percentage. This means
%that the plot is a joint probability density function plot of wind speed
%and wave height
figure(7)
[q,p] = meshgrid(u1bins,H1bins);
pdfz1=hist3([X],'Edges', {u1bins,H1bins});
pdfz1=pdfz1'/sum(r1(:));
surf(p,q,pdfz1)
xlabel('Hs, Significant Wave Height [m]');
ylabel('u, Wind speed [m/s]');
zlabel('Joint probability');
title('Joint probability density function')

%I interpolate the previous plot to make a smoother surface
figure(8)
H1bins1 = 0:0.2:15;
u1bins1 = 0:0.2:50;
[q1,p1] = meshgrid(u1bins1,H1bins1);
ZI = interp2(q,p,pdfz1,q1,p1);
surf(p1,q1,ZI+0.01,'EdgeColor','none')
xlabel('Hs, Significant Wave Height [m]');
ylabel('u, Wind speed [m/s]');
zlabel('Joint probability');
title('Joint probability density function')
set(gcf,'renderer','opengl');
set(get(gca,'child'),'FaceColor','interp','CDataMode','auto');


%% ENERGY PRODUCTION BY WIND TURBINE
%% Power curve

%The power curve from Enercons E-126, NREL 5 MW or Vestas 3 MW is created
xdata = 0:1:50; %Wind speeds
%Enercon's E-126 (7580 kW) with storm control
%ydata = [0.0 0.0 55.0 175.0 410.0 760.0 1250 1900 2700 3750 4850 5750 6500 7000 7350 7500 7580 7580 7580 7580 7580 7580 7580 7580 7580 6000 5000 4000 3000 2000 1000 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0];

%Vestas 3000 MW
%ydata = [0 0 0 0 75 187 348 574 875 1257 1688 2118 2514 2817 2958 2994 2999 3000 3000 3000 3000 3000 3000 3000 3000 3000 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0];

%NREL 5 MW
ydata = [0 0 0 40.5 177.7 403.9 737.6 1187.2 1771.1 2518.6 3448.4 4562.5 5000.0 5000.0 5000.0 5000.0 5000.0 5000.0 5000.0 5000.0 5000.0 5000.0 5000.0 5000.0 5000.0 5000.0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0];

%The power curve is plotted
Effektkurve = [xdata;ydata]';
figure(13)
plot(xdata,ydata)
xlabel('u_h_u_b, Wind speed in hub height[m/s]');
ylabel('Power output [kW]');
title('Power curve')




%% Power time series
%The power time series is made from the wind speed series input in the
%power curve and the time series is plotted
effekttidsserie = interp1(Effektkurve(:,1),Effektkurve(:,2),u);
effekttidsserie(effekttidsserie<40.5)=0; % Here I remove all wind power created under cut-in speed

%The total amount of energy produced is the sum of the power time series
time = 0.33333; %This assumes that the energy produced is power x time, where time is 20 minutes = 0.33333 hours
Energiutencutoff = sum(effekttidsserie)*time; %Energy in kWh


%%  Cut-out wind speed

%The interpolation function is not perfect, so I create a new variable which removes all power made from wind speeds
%above 25 m/s.
Pcutoffvind=effekttidsserie;
cutoffvind = 25.0; %This needs to be an integer
T = find(cutoffvind<=u & u< round(max(u)+1));
Q = Pcutoffvind(T); %The variable Q shows the power of the selected index T
Pcutoffvind(T) = 0; %Put all powers at index T equal to zero
figure(40)
plot(k, Pcutoffvind,'r')
datetick
xlabel('Time');
ylabel('Power [kW]');
title('Time series of power output without cut-out wave height')

Energimedcutoffvind = sum(Pcutoffvind)*time;


%% Years of valid data, full load hours and AEP
aar = length(H)/(365*24*3); %Number of valid years
Energiperaar = Energimedcutoffvind/(aar); %Energy production 
brukstimer = Energiperaar/(max(ydata)); %Full load hours
capacityfactor=(brukstimer/8760); % Capacity factor


%% High wind hysteresis (cut-out and re cut-in) control strategy (used for
%% Vestas V90)

Pcutoffvindrecutinn=Pcutoffvind;
recutinnvind=20; %Re cut-in wind speed
%Put all power between cut-out wind speed and re-cut in wind speed to zero
for i=2:length(u)
    if (Pcutoffvindrecutinn(i-1)==0 && u(i)>recutinnvind);
        Pcutoffvindrecutinn(i)=0;
    end
end

Energimedcutoffvindrecutinn = sum(Pcutoffvindrecutinn)*time;  % Energy production with re-cut inn wind speed
prosent = (Energimedcutoffvindrecutinn-Energimedcutoffvind)/(Energimedcutoffvind);


%%  Energy production vs. wind speed histogram
u=u';
%In order to make histograms, I need to extract wind speeds for the different power produced
for i=1:round(max(u)+1);
    n{i} = find((i-1)<=u & u<i);
end


%Since n is a array, I need to convert it to a matrix
maxSize = max(cellfun(@numel,n));    %# Get the maximum vector size
fcn = @(x) [x nan(1,maxSize-numel(x))];  %# Create an anonymous function
rmat = cellfun(fcn,n,'UniformOutput',false);  %# Pad each cell with NaNs
rmat = vertcat(rmat{:});
rmat(isnan(rmat))=1; %I have deliberality put u(1) equal to zero!!!!!!

%I extract the power produced for different wind speeds
Pdifferentwindspeeds = Pcutoffvind(rmat(:,:));
for i=1:(round(max(u)));
rtu(i) =sum(Pdifferentwindspeeds(i,:));
end

%I create a histogram of the energy produced at different wind speeds
i=0.5:1:round(max(u));
figure(50)
hold on
bar(i,rtu,1,'r')
xlabel('Wind speed [m/s]');
ylabel('Energy output [kWh]');
title('Energy production vs. wind speed histogram')
Energihistogram_utencutoff = sum(rtu)*time;

%I sum up the energy output under the cut-off wind speed and plot it in the
%same graph, making the red part showing the lost energy
Vcutoff = cutoffvind-0.1; %I need to extract 0.1 to avoid including cutoffspeed
Histogram_cutoffvind = (rtu(1:(Vcutoff+1))); %+1 is because rtu(2) means wind speeds of 1 m/s
j= 0.5:1:(0.5+Vcutoff);
bar(j,Histogram_cutoffvind,1,'b')
hold off

%I sum up the values in the blue part of the histogram and calculate the
%loss of energy in kWh and percentage
Energihistogram_cutoffvind=sum(Histogram_cutoffvind)*time;
Taptenergi_cutoffvind = Energihistogram_utencutoff - Energihistogram_cutoffvind;
Taptenergi_cutoffvind_prosent = Taptenergi_cutoffvind/Energihistogram_cutoffvind;




%% INTRODUCING THE CUT OUT WAVE HEIGHT

Pcutoffbolge=Pcutoffvind;
figure(41)
B =find(cutoffbolge<=H & H< round(max(H)+1));
Pcutoffbolge(B)=0; %Put all power at index B equal to zero
plot(k, Pcutoffbolge,'g')
plot(k, Pcutoffvind,'r')
xlabel('Time');
ylabel('Power [kW]');
title('Time series of power output with cut-out wave height')
Energimedcutoffbolge = sum(Pcutoffbolge)*time;
Tapt_energi_prosent_cutoffbolge= (Energimedcutoffvind-Energimedcutoffbolge)/(Energimedcutoffvind);



%% Power series vs. wave distribution

%If I want to remove power produced above the cutoff wave height (thus making both cutoff
%wind speed and cutoff wave heights count), I need to use Pcutoffvind
%instead of effekttidsserie
H=H';
for i=1:round(max(H)+1);
    b{i} = find((i-1)<=H & H<i);
end
maxSize = max(cellfun(@numel,b));    %# Get the maximum vector size
fcn1 = @(x) [x nan(1,maxSize-numel(x))];  %# Create an anonymous function
bmat = cellfun(fcn1,b,'UniformOutput',false);  %# Pad each cell with NaNs
bmat = vertcat(bmat{:});
bmat(isnan(bmat))=1;

%Power produced at different wave heigts
Pdifferentwaveheights=Pcutoffvind(bmat(:,:));
for j=1:round(max(H)+1);
    btu(j) = sum(Pdifferentwaveheights(j,:));
end

%Histogram for energy output of different wave heights
j= 0.5:1:round(max(H)+1);
figure(51)
hold on
bar(j,btu,1,'r')
xlabel('H_s, significant wave height [m]');
ylabel('Energy output [kWh]');
title('Energy production vs. wave height histogram')
Energihistogram_utencutoff2 = sum(btu)*time;

%I sum up the energy output for wave heights under the cut-off height and
%plot the result in the same graphs. the red part is lost energy
Hcutoff = cutoffbolge-0.1;;
Histogram_cutoffbolge = (btu(1:(Hcutoff+1)));
j= 0.5:1:(0.5+Hcutoff);
bar(j,Histogram_cutoffbolge,1,'b')

%I sum up the values in the blue part of the histogram and calculate the
%loss of energy in kWh and percentage due to cutting off
Energihistogram_cutoffbolge=sum(Histogram_cutoffbolge)*time;
Taptenergi_cutoffbolge = Energihistogram_utencutoff2 - Energihistogram_cutoffbolge;
Taptenergi_cutoffbolge_prosent = Taptenergi_cutoffbolge/Energihistogram_utencutoff2; %Energy loss due to the cut-out wave height
Taptenergi_cutoffbolge_2=abs((Energimedcutoffbolge-Energimedcutoffvind)/(Energimedcutoffvind));


%%  Ideal energy loss in percentage for different cut-out wave heights
E = sum(btu);
Q = btu/E;
for i=1:length(Q);
    R(1) = 1;
    R(i+1) = 1 - (sum(Q(1:i)));
end
r = 0:length(Q);
figure(52)
plot(r,R)
xlabel('H_s,cut-out [m]');
ylabel('Energy loss [fraction]');
title('Ideal energy loss at different values of the cut-out wave height')

Taptenergi_kumulativ_prosent = R(cutoffbolge+1); %Energy loss due the cut-out wave height calculated by the histogram







%% STRATEGIES FOR RECONFIGURATION INTO SURVIVAL MODE
%Because the wave height can vary considerably and sometimes around the
%cut-out wave height, this would lead to the mooring lines to go up and
%down (or another survival mode system).
% To illustrate this, I calculate how many times and how tight it is
%between the different up and down reconfiguration.

limit = cutoffbolge-0.000001;
for i=2:length(H)
    if (H(i)>=limit) && (H(i-1)<limit) 
        D(i+1)= i; %D is the point where the wave height goes over cut-off wave height
    elseif (H(i)<=limit) && (H(i-1)>limit) 
        C(i+1)=i;%C is the point where the wave height goes over cut-off wave height
    end
end
D(D==0)=[];
length(D);
C(C==0)=[];
Y1 = [D' C'];
strategi0__antallganger_opp_og_ned_=length(Y1) %Number of total reconfigurations ideally
strategi0__tid_mellom_ganger = (C-D)'

%It is obvious that the time interval between cut-out and re-start can
%sometimes be very short (20 or 40 minutes). This is the reason why we must
%find a strategy for when the turbine reconfigures the mooring lines.


%%%%%%%%%%%%%%%%%%%%%THE DIFFERENT STRATEGIES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%There are different strategies to consider to decide whether to turn of
%the wind turbine or not. Here I have considered several possibilities:


%1. Strategy N0.1 - introducing re-cut-inn wave height
% if the wave height is over cut-off more than one time
%in a row, the turbine cuts off. The turbine will stay shut down until the
%wave height reaches a lower level, for example 2 metres under cut-off.
%   a) 20 minutes
%   b) 40 minutes
%   c) 60 minutes


%2. Strategy N0.2 - using forecast with uncertainty with different reconfiguration time
%   a) 20 minutes
%   b) 40 minutes
%   c) 60 minutes

%3. Making a perfect forecast by correcting forecast with measurements
%Only assumed reconfiguration time of 20 minutes






%% STRATEGY NO. 1: RE CUT-IN WAVE HEIGHT

 
%Explanation of strategy no 1


Pcutoffbolgerecutinn = Pcutoffbolge;
z=H;

for i=2:length(z)
    if (Pcutoffbolgerecutinn(i-1)==0 && z(i-1)>cutoffbolge && H(i)>recutinn);
        Pcutoffbolgerecutinn(i)=0;
        z(i)=cutoffbolge+0.0001;
        fjernet(i)=1; %%%%%%%%%%%%%%%%%%%%%%%%%%shows what has been removed
    end
end
strategi1_Energimedcutoffbolgerecutinn = sum(Pcutoffbolgerecutinn)*time;
strategi1_Tapt_energi_prosent= (Energimedcutoffvind - strategi1_Energimedcutoffbolgerecutinn)/(Energimedcutoffvind);

%Find removed cells and where they go up and down
for i=2:length(fjernet)
     if(abs(fjernet(i)-fjernet(i-1)==1))
         up_strategi1(i)=i;
     elseif(fjernet(i-1)-fjernet(i)==1)
         down_strategi1(i)=i;
     end
end
up_strategi1(up_strategi1==0)=[];
down_strategi1(down_strategi1==0)=[];

strategi1_antallgangeropp = length(up_strategi1);
strategi1_tidspunkter_for_shutdownogrestart = [up_strategi1(1:length(down_strategi1))' down_strategi1']
strategi1_maxbolge = [ H(strategi1_tidspunkter_for_shutdownogrestart(:,1)-1)' H(strategi1_tidspunkter_for_shutdownogrestart(:,1))' H(strategi1_tidspunkter_for_shutdownogrestart(:,1)+1)' H(strategi1_tidspunkter_for_shutdownogrestart(:,1)+2)' H(strategi1_tidspunkter_for_shutdownogrestart(:,1)+3)' ]
%Check if they are too close
for i=1:length(up_strategi1)-1
    ertert(1)=0;
    if(abs(down_strategi1(i)-up_strategi1(i))<5) %%% five = 20*5= 100 minutes, which is over the reconfiguration time
        ertert(i)=i;
    end
end
ertert(ertert==0)=[];
strategi1_for_kort_tid_mellomganger_=(down_strategi1(ertert)-up_strategi1(ertert));



%% 1a)  Reconfiguration time 20 minutes

Pcutoffbolgerecutinn_1a = Pcutoffbolge;
z_1a=H;
for i=2:length(z)-1
    if (Pcutoffbolgerecutinn_1a(i-1)==0 && z(i-1)>cutoffbolge && H(i)>recutinn);
        Pcutoffbolgerecutinn_1a(i)=0;
        Pcutoffbolgerecutinn_1a(i+1)=0;
        z_1a(i)=cutoffbolge+0.0001;
        fjernet_1a(i)=1; %%%%%%%%%%%%%%%%%%%%%%%%%%shows what has been removed
    end
end
strategi1a_Energimedcutoffbolgerecutinn = sum(Pcutoffbolgerecutinn_1a)*time;
strategi1a_Tapt_energi_prosent= (Energimedcutoffvind - strategi1a_Energimedcutoffbolgerecutinn)/(Energimedcutoffvind);
strategi1a_maxbolge = max(max(strategi1_maxbolge(:,1:3)))


%% 1b)  Reconfiguration time 40 minutes

Pcutoffbolgerecutinn_1b = Pcutoffbolge;
z_1b=H;
for i=2:length(z)-1
    if (Pcutoffbolgerecutinn_1b(i-1)==0 && z(i-1)>cutoffbolge && H(i)>recutinn);
        Pcutoffbolgerecutinn_1b(i)=0;
        Pcutoffbolgerecutinn_1b(i+1)=0;
        Pcutoffbolgerecutinn_1b(i+2)=0;
        z_1b(i)=cutoffbolge+0.0001;
        fjernet_1b(i)=1; %%%%%%%%%%%%%%%%%%%%%%%%%%shows what has been removed
    end
end
strategi1b_Energimedcutoffbolgerecutinn = sum(Pcutoffbolgerecutinn_1b)*time;
strategi1b_Tapt_energi_prosent= (Energimedcutoffvind - strategi1b_Energimedcutoffbolgerecutinn)/(Energimedcutoffvind);
strategi1b_maxbolge = max(max(strategi1_maxbolge(:,1:4)))


%% 1c)  Reconfiguration time 60 minutes

Pcutoffbolgerecutinn_1c = Pcutoffbolge;
z_1c=H;
for i=2:length(z)-1
    if (Pcutoffbolgerecutinn_1c(i-1)==0 && z(i-1)>cutoffbolge && H(i)>recutinn);
        Pcutoffbolgerecutinn_1c(i)=0;
        Pcutoffbolgerecutinn_1c(i+1)=0;
        Pcutoffbolgerecutinn_1c(i+2)=0;
        Pcutoffbolgerecutinn_1c(i+3)=0;
        z_1c(i)=cutoffbolge+0.0001;
        fjernet_1c(i)=1; %%%%%%%%%%%%%%%%%%%%%%%%%%shows what has been removed
    end
end
strategi1c_Energimedcutoffbolgerecutinn = sum(Pcutoffbolgerecutinn_1c)*time;
strategi1c_Tapt_energi_prosent= (Energimedcutoffvind - strategi1c_Energimedcutoffbolgerecutinn)/(Energimedcutoffvind);
strategi1c_maxbolge = max(max(strategi1_maxbolge(:,1:5)))




%% STRATEGY NO. 2 - COMBINING MEASUREMENTS AND FORECAST

% Strategy No. 2 is an attempt to combine measurements and forecast 
% assuming that the forecast has some uncertainty. A forecast with
% uncertainty is synthesized with:

for i=1:length(H)
    H_forecast(i)=H(i)+rand(1)-0.25; %Making forecast with +0.25 m bias in average
end


% The turbine shuts down and the mooring lines are reconfigured when:

% 1. The wave height in step i (H(i)) is above cutoff    OR when

% 2. The wave height 20/40/60 minutes ahead in time, depending on the reconfiguration time,
% is above cut-out wave height minus a safety limit (Hcutoff - safety limit).
% If the cut-out wave height is 7 m and the safety limit is 0.5 m, 
% the turbine cuts out at  forecast of 6.5 m.


% The turbine re cut-in when every forecast three hours ahead in time shows
% a Hs below the (Hcutoff - safety limit) AND when Hs < Hs,cut-out



%% 2a)  Reconfiguration time 20 minutes



strategi2a_slingring=0.5 %Safety limit
strategi2a_rekonfigurasjonstid=1 % 1 equals 20 minutes
strategi2a_tilstand(1)=11 %11 means that the turbine is operational, 9 means that the turbine turns off


for i=2:length(H)-10
    if(strategi2a_tilstand(i-1)==11) 
        if (H(i)>cutoffbolge || (H_forecast(i+2*strategi2a_rekonfigurasjonstid)>cutoffbolge)) 
             strategi2a_tilstand(i)=9; 
        else
            strategi2a_tilstand(i)=11; 
        end
    elseif(strategi2a_tilstand(i-1)==9) 
         if (H(i)<cutoffbolge && H_forecast(i+1)<cutoffbolge-strategi2a_slingring && H_forecast(i+2)<cutoffbolge-strategi2a_slingring && H_forecast(i+3)<cutoffbolge-strategi2a_slingring && H_forecast(i+4)<cutoffbolge-strategi2a_slingring && H_forecast(i+5)<cutoffbolge-strategi2a_slingring && H_forecast(i+6)<cutoffbolge-strategi2a_slingring && H_forecast(i+7)<cutoffbolge-strategi2a_slingring && H_forecast(i+8)<cutoffbolge-strategi2a_slingring && H_forecast(i+9)<cutoffbolge-strategi2a_slingring) % Hvis framtidig hyde er under 6.7 m,
            strategi2a_tilstand(i)=11; 
         else
             strategi2a_tilstand(i)=9; 
         end
    end
end



%Finding how often the reconfiguration system goes up and dow
for i=2:length(H)-10
    if(strategi2a_tilstand(i)-strategi2a_tilstand(i-1)==2)
        strategi2a_down(i)=i;
    elseif(strategi2a_tilstand(i)-strategi2a_tilstand(i-1)==-2)
        strategi2a_up(i)=i;
    end
end
strategi2a_up(strategi2a_up==0)=[];
strategi2a_down(strategi2a_down==0)=[];

strategi2a_avstand = [strategi2a_down-strategi2a_up]'; %The distance (in time steps) for when the turbine goes up and down
strategi2a_tilstanddiagram = [strategi2a_up'  strategi2a_down' strategi2a_avstand];
strategi2a_minsteavstand=min(strategi2a_avstand); %The minimum distance between going up and down
strategi2a_antallgangeroppogned=length(strategi2a_up)/aar; %How many times the turbine goes up and down a year


%To find out when the turbine is operating, i have to put in a delay of the
%reconfiguration time to calculate the correct energy loss
strategi2a_energitilstand=strategi2a_tilstand;
for i=1:length(strategi2a_down)
   strategi2a_energitilstand(strategi2a_up(i):strategi2a_down(i)+strategi2a_rekonfigurasjonstid)=9;
end

%Energy production at this strategy
P_strategi2a=Pcutoffbolge;
strategi2a_tidspunkter_turbin_ikke_i_drift = find(strategi2a_energitilstand==9)+1; %+1 is because the turbine will first turn off after the condition is changed
P_strategi2a(strategi2a_tidspunkter_turbin_ikke_i_drift)=0; %removes all power produced at these times
strategi2a_Energi_produsert=sum(P_strategi2a)*time; 
strategi2a_Tapt_energi_prosent=(Energimedcutoffvind-strategi2a_Energi_produsert)/(Energimedcutoffvind);

%Waves experienced by the turbine in different situations
strategi2a_maxbolge_in_operation = max(H(find(strategi2a_tilstand==11))) %Maximum wave when the turbine is in operation
strategi2a_maxbolge_during_reconfiguration = max(max([H(strategi2a_up)' H(strategi2a_up+1)'])) % delay of 1 due to the reconfiguration time
strategi2a_wave_in_survival_mode = max(H(find (P_strategi2a(1:length(strategi2a_tilstand)) == 0 & strategi2a_tilstand'==9))); % Maximum wave in survival mode






%% 2b)  Reconfiguration time 40 minutes 
strategi2b_slingring=0.5
strategi2b_rekonfigurasjonstid=2 % 2 equals 40 minutes
strategi2b_tilstand(1)=11

for i=2:length(H)-10
    if(strategi2b_tilstand(i-1)==11) %11 means that the turbine is running
        if (H(i)>cutoffbolge || (H_forecast(i+2*strategi2b_rekonfigurasjonstid)>cutoffbolge)) % Hvis nvrende hyde er over 7 meter eller framtidig hyde ( 20 og 40 min) er over 6.7 m
             strategi2b_tilstand(i)=9; % 9 means that the turbine turns off
        else
            strategi2b_tilstand(i)=11; % Hvis ikke, bli p 11
        end
    elseif(strategi2b_tilstand(i-1)==9) %Turbinen er avskrudd
         if (H(i)<cutoffbolge && H_forecast(i+1)<cutoffbolge-strategi2b_slingring && H_forecast(i+2)<cutoffbolge-strategi2b_slingring && H_forecast(i+3)<cutoffbolge-strategi2b_slingring && H_forecast(i+4)<cutoffbolge-strategi2b_slingring && H_forecast(i+5)<cutoffbolge-strategi2b_slingring && H_forecast(i+6)<cutoffbolge-strategi2b_slingring && H_forecast(i+7)<cutoffbolge-strategi2b_slingring && H_forecast(i+8)<cutoffbolge-strategi2b_slingring && H_forecast(i+9)<cutoffbolge-strategi2b_slingring) % Hvis framtidig hyde er under 6.7 m,
            strategi2b_tilstand(i)=11; % G p igjen
         else
             strategi2b_tilstand(i)=9; %Hvis ikke, forbli avskrudd
         end
    end
end



%Finding how often the reconfiguration system goes up and dow
for i=2:length(H)-10
    if(strategi2b_tilstand(i)-strategi2b_tilstand(i-1)==2)
        strategi2b_down(i)=i;
    elseif(strategi2b_tilstand(i)-strategi2b_tilstand(i-1)==-2)
        strategi2b_up(i)=i;
    end
end
strategi2b_up(strategi2b_up==0)=[];
strategi2b_down(strategi2b_down==0)=[];

strategi2b_avstand = [strategi2b_down-strategi2b_up]'; %The distance (in time steps) for when the turbine goes up and down
strategi2b_tilstanddiagram = [strategi2b_up'  strategi2b_down' strategi2b_avstand];
strategi2b_minsteavstand=min(strategi2b_avstand); %The minimum distance between going up and down
strategi2b_antallgangeroppogned=length(strategi2b_up)/aar; %How many times the turbine goes up and down a year


%To find out when the turbine is operating, i have to put in a delay of the
%reconfiguration time to calculate the correct energy loss
strategi2b_energitilstand=strategi2b_tilstand;
for i=1:length(strategi2b_down)
   strategi2b_energitilstand(strategi2b_up(i):strategi2b_down(i)+strategi2b_rekonfigurasjonstid)=9;
end

%Energy production at this strategy
P_strategi2b=Pcutoffbolge;
strategi2b_tidspunkter_turbin_ikke_i_drift = find(strategi2b_energitilstand==9)+1; %+1 is because the turbine will first turn off after the condition is changed
P_strategi2b(strategi2b_tidspunkter_turbin_ikke_i_drift)=0; %removes all power produced at these times
strategi2b_Energi_produsert=sum(P_strategi2b)*time; 
strategi2b_Tapt_energi_prosent=(Energimedcutoffvind-strategi2b_Energi_produsert)/(Energimedcutoffvind);

%Waves experienced by the turbine in different situations
strategi2b_maxbolge_in_operation = max(H(find(strategi2b_tilstand==11))) %Maximum wave when the turbine is in operation
strategi2b_maxbolge_during_reconfiguration = max(max([H(strategi2b_up)' H(strategi2b_up+1)' H(strategi2b_up+2)'])) % delay of 2 due to the reconfiguration time
strategi2b_wave_in_survival_mode = max(H(find (P_strategi2b(1:length(strategi2b_tilstand)) == 0 & strategi2b_tilstand'==9))); % Maximum wave in survival mode








%% 2c)  Reconfiguration time 60 minutes
strategi2c_slingring=0.5
strategi2c_rekonfigurasjonstid=3 % 3 equals 60 minutes
strategi2c_tilstand(1)=11

for i=2:length(H)-10
    if(strategi2c_tilstand(i-1)==11) %11 means that the turbine is running
        if (H(i)>cutoffbolge || (H_forecast(i+2*strategi2c_rekonfigurasjonstid)>cutoffbolge)) % Hvis nvrende hyde er over 7 meter eller framtidig hyde ( 20 og 40 min) er over 6.7 m
             strategi2c_tilstand(i)=9; % 9 means that the turbine turns off
        else
            strategi2c_tilstand(i)=11; % Hvis ikke, bli p 11
        end
    elseif(strategi2c_tilstand(i-1)==9) %Turbinen er avskrudd
         if (H(i)<cutoffbolge && H_forecast(i+1)<cutoffbolge-strategi2c_slingring && H_forecast(i+2)<cutoffbolge-strategi2c_slingring && H_forecast(i+3)<cutoffbolge-strategi2c_slingring && H_forecast(i+4)<cutoffbolge-strategi2c_slingring && H_forecast(i+5)<cutoffbolge-strategi2c_slingring && H_forecast(i+6)<cutoffbolge-strategi2c_slingring && H_forecast(i+7)<cutoffbolge-strategi2c_slingring && H_forecast(i+8)<cutoffbolge-strategi2c_slingring && H_forecast(i+9)<cutoffbolge-strategi2c_slingring) % Hvis framtidig hyde er under 6.7 m,
            strategi2c_tilstand(i)=11; % G p igjen
         else
             strategi2c_tilstand(i)=9; %Hvis ikke, forbli avskrudd
         end
    end
end


%Finding how often the reconfiguration system goes up and dow
for i=2:length(H)-10
    if(strategi2c_tilstand(i)-strategi2c_tilstand(i-1)==2)
        strategi2c_down(i)=i;
    elseif(strategi2c_tilstand(i)-strategi2c_tilstand(i-1)==-2)
        strategi2c_up(i)=i;
    end
end
strategi2c_up(strategi2c_up==0)=[];
strategi2c_down(strategi2c_down==0)=[];

strategi2c_avstand = [strategi2c_down-strategi2c_up]'; %The distance (in time steps) for when the turbine goes up and down
strategi2c_tilstanddiagram = [strategi2c_up'  strategi2c_down' strategi2c_avstand];
strategi2c_minsteavstand=min(strategi2c_avstand); %The minimum distance between going up and down
strategi2c_antallgangeroppogned=length(strategi2c_up)/aar; %How many times the turbine goes up and down a year


%To find out when the turbine is operating, i have to put in a delay of the
%reconfiguration time to calculate the correct energy loss
strategi2c_energitilstand=strategi2c_tilstand;
for i=1:length(strategi2c_down)
   strategi2c_energitilstand(strategi2c_up(i):strategi2c_down(i)+strategi2c_rekonfigurasjonstid)=9;
end

%Energy production at this strategy
P_strategi2c=Pcutoffbolge;
strategi2c_tidspunkter_turbin_ikke_i_drift = find(strategi2c_energitilstand==9)+1; %+1 is because the turbine will first turn off after the condition is changed
P_strategi2c(strategi2c_tidspunkter_turbin_ikke_i_drift)=0; %removes all power produced at these times
strategi2c_Energi_produsert=sum(P_strategi2c)*time; 
strategi2c_Tapt_energi_prosent=(Energimedcutoffvind-strategi2c_Energi_produsert)/(Energimedcutoffvind);

%Waves experienced by the turbine in different situations

strategi2c_maxbolge_in_operation = max(H(find(strategi2b_tilstand==11))) %Maximum wave when the turbine is in operation
strategi2c_maxbolge_during_reconfiguration  = max(max([H(strategi2c_up)' H(strategi2c_up+1)' H(strategi2c_up+2)' H(strategi2c_up+3)'])) % delay of 3 due to the reconfiguration time
strategi2c_wave_in_survival_mode = max(H(find (P_strategi2c(1:length(strategi2c_tilstand)) == 0 & strategi2c_tilstand'==9))); % Maximum wave in survival mode





%% STRATEGY NO 3: PERFECT FORECAST

%Strategy 3 is just strategy 2 without a safety limit and assuming a
%perfect forecast. For each step i of H, previous values of H are
%considered to be measurements and data ahead in time are considered to be
%forecast.

strategi3_slingring=0.0
strategi3_rekonfigurasjonstid=1 % 1 equals 60 minutes
strategi3_tilstand(1)=11

for i=2:length(H)-10
    if(strategi3_tilstand(i-1)==11) %11 means that the turbine is running
        if (H(i)>cutoffbolge || (H(i+2*strategi3_rekonfigurasjonstid)>cutoffbolge-strategi3_slingring)) % Hvis nvrende hyde er over 7 meter eller framtidig hyde ( 20 og 40 min) er over 6.7 m
             strategi3_tilstand(i)=9; % 9 means that the turbine turns off
        else
            strategi3_tilstand(i)=11; % Hvis ikke, bli p 11
        end
    elseif(strategi3_tilstand(i-1)==9) %Turbinen er avskrudd
         if (H(i)<cutoffbolge && H(i+1)<cutoffbolge-strategi3_slingring && H(i+2)<cutoffbolge-strategi3_slingring && H(i+3)<cutoffbolge-strategi3_slingring && H(i+4)<cutoffbolge-strategi3_slingring && H(i+5)<cutoffbolge-strategi3_slingring && H(i+6)<cutoffbolge-strategi3_slingring && H(i+7)<cutoffbolge-strategi3_slingring && H(i+8)<cutoffbolge-strategi3_slingring && H(i+9)<cutoffbolge-strategi3_slingring) % Hvis framtidig hyde er under 6.7 m,
            strategi3_tilstand(i)=11; % G p igjen
         else
             strategi3_tilstand(i)=9; %Hvis ikke, forbli avskrudd
         end
    end
end


%Finding how often the reconfiguration system goes up and dow
for i=2:length(H)-10
    if(strategi3_tilstand(i)-strategi3_tilstand(i-1)==2)
        strategi3_down(i)=i;
    elseif(strategi3_tilstand(i)-strategi3_tilstand(i-1)==-2)
        strategi3_up(i)=i;
    end
end
strategi3_up(strategi3_up==0)=[];
strategi3_down(strategi3_down==0)=[];

strategi3_avstand = [strategi3_down-strategi3_up]'; %The distance (in time steps) for when the turbine goes up and down
strategi3_tilstanddiagram = [strategi3_up'  strategi3_down' strategi3_avstand];
strategi3_minsteavstand=min(strategi3_avstand); %The minimum distance between going up and down
strategi3_antallgangeroppogned=length(strategi3_up)/aar; %How many times the turbine goes up and down a year


%To find out when the turbine is operating, i have to put in a delay of the
%reconfiguration time to calculate the correct energy loss
strategi3_energitilstand=strategi3_tilstand;
for i=1:length(strategi3_down)
   strategi3_energitilstand(strategi3_up(i):strategi3_down(i)+strategi3_rekonfigurasjonstid)=9;
end

%Energy production at this strategy
P_strategi3=Pcutoffbolge;
strategi3_tidspunkter_turbin_ikke_i_drift = find(strategi3_energitilstand==9)+1; %+1 is because the turbine will first turn off after the condition is changed
P_strategi3(strategi3_tidspunkter_turbin_ikke_i_drift)=0; %removes all power produced at these times
strategi3_Energi_produsert=sum(P_strategi3)*time; 
strategi3_Tapt_energi_prosent=(Energimedcutoffvind-strategi3_Energi_produsert)/(Energimedcutoffvind);

%Waves experienced by the turbine in different situations
strategi3_maxbolge_in_operation = max(H(find(strategi3_tilstand==11))) %Maximum wave when the turbine is in operation
strategi3_maxbolge_during_reconfiguration = max(max([H(strategi3_up)' H(strategi3_up+1)'])) % delay of 1 due to the reconfiguration time
strategi3_wave_in_survival_mode = max(H(find (P_strategi3(1:length(strategi3_tilstand)) == 0 & strategi3_tilstand'==9))); % Maximum wave in survival mode

strategi2a_maxbolge_in_operation = max(H(find(strategi2a_tilstand==11))) %Maximum wave when the turbine is in operation
strategi2a_maxbolge_during_reconfiguration = max(max([H(strategi2a_up)' H(strategi2a_up+1)'])) % delay of 1 due to the reconfiguration time
strategi2a_wave_in_survival_mode = max(H(find (P_strategi2a(1:length(strategi2a_tilstand)) == 0 & strategi2a_tilstand'==9))); % Maximum wave in survival mode




